package org.smartjava.scalatra

import grizzled.slf4j.Logger
import org.scalatra._
import scalate.ScalateSupport
import net.liftweb.json.compact
import net.liftweb.json.render
import net.liftweb.json.JsonDSL._
import net.liftweb.json.Serialization.{read, write}
import org.smartjava.scalatra.repository.ItemRepository
import net.liftweb.json.Serialization
import net.liftweb.json.NoTypeHints
import org.scalatra.Ok
import org.scalatra.NotFound
import org.smartjava.scalatra.repository.BidRepository
import org.scalatra.Created
import scala.collection.immutable.Map
import org.smartjava.scalatra.model.Bid

/**
 * This is just a simple servlet (the entry point for our API). For this
 * part of the tutorial, we create a simple API that allows you to bid on
 * items, retrieve item information, and retrieve seller information.
 */
class HelloScalatraServlet extends ScalatraServlet with ScalateSupport {
  
  // simple logger
  val logger = Logger(classOf[HelloScalatraServlet]);

  // repo stores our items
  val itemRepo = new ItemRepository;
  val bidRepo = new BidRepository;
  
  // implicit value for json serialization format
  implicit val formats = Serialization.formats(NoTypeHints);
  
  /**
   * Handles get based on items id. This operation doesn't have a specific
   * media-type since we're doing a simple GET without content. This operation
   * returns an item of the type application/vnd.smartbid.item+json
   */
  get("/items/:id") {
    // set the result content type
    contentType = "application/vnd.smartbid.item+json"
      
    // convert response to json and return as OK
    itemRepo.get(params("id").toInt) match {
      case Some(x) => Ok(write(x));
      case None => NotFound("Item with id " + params("id") + " not found");
    }
  }
  
  /**
   * To make a bid, we need to post a message to a specific url. Based on
   * the information from the url and the content a bid can be made.
   * 
   * {
   * "forItem":123,
   * "minimum":20,
   * "maximum":10,
   * "currency":"GBP",
   * "bidder":"jdirksen",
   * "date":1347269593301
   * }
   * 
   * This operation will always create a new bid. If this is called twice, two bids will
   * be created. To alter a bid, use PUT.
   */
  post("/items/:id/bid", request.getContentType == "application/vnd.smartbid.bid+json") {
    contentType = "application/vnd.smartbid.bid+json"
	var createdBid = bidRepo.create(read[Bid](request.body));
	Created(write(createdBid), Map("Location"->("/users/" + createdBid.bidder + "/bids/"+createdBid.id.get)));
  }
  
  /**
   * Route that matches retrieval of bids
   */
  get("/users/:user/bids/:bid") {
    contentType = "application/vnd.smartbid.bid+json"
    
      // convert response to json and return as OK
    implicit val formats = Serialization.formats(NoTypeHints);
    bidRepo.get(params("bid").toInt,params("user")) match {
      case Some(x) => Ok(write(x));
      case None => NotFound("Bid with id " + params("bid") + " not found for user: " + params("user") );
    }
  }
  
  /**
   * Standard result for when no resource can be found
   */
  notFound {
    // remove content type in case it was set through an action
    contentType = null 
    // Try to render a ScalateTemplate if no route matched
    findTemplate(requestPath) map { path =>
      contentType = "text/html"
      layoutTemplate(path)
    } orElse serveStaticResource() getOrElse resourceNotFound() 
  }
}
